class MachineThroughputDashboard < RxModule::Dashboards::DualAxisChart
  include MachineDashboardHelpers
  include DataHelpers
  include KpiHelpers
  include BucketCalculator

  expose('machine_throughput', 'Machine Throughput', 'View the number of successful job states per hour for the selected machine(s)')
  ui_hint(type: 'type_override')
  ui_hint(type_override: 'line')
  ui_hint(options_override: {
    tooltip: {
      title: {
        formatter: 'machine_throughput'
      }
    },
    legend: {
      offsetY: 5,
      formatter: 'machine_throughput'
    },
    stroke: {
      curve: 'straight'
    },
    markers: {
      size: 4
    },
    xaxis: {
      title: {
        text: 'machine_throughput'
      }
    },
    yaxis: {
      min: 0,
      title: {
        text: 'machine_throughput'
      }
    }
  })
  ui_hint(data_labels_toggle: true)
  ui_hint(dash_array_toggle: true)
  ui_hint(show_legend_toggle: true)
  targets("Machine", "The machine(s) to track", -> { machine_targets }, target_type: nil, custom_targets: false)
  parameter(:buckets, "Hours to display", "How many hours to display", :string, required: true, value_list: -> { total_time })
  parameter(:target_throughput, "Machine Throughput Target", "Display target machine throughput on chart", :integer, default: 0, required: false, min: 0)
  parameter(:offset, "Start at mid-hour?", "When enabled, column starts with start at the middle of the hour (ex: 1:30, 2:30, etc.)", :boolean, default: false)
  parameter(:split_machine_groups, "Split machine group into individual machines?", "When enabled, machine groups will be split into individual machines", :boolean, default: false)
  parameter(:data_labels_toggle, "Display throughput values on chart?", "When enabled, machine throughput values will be displayed for each point on the chart", :boolean, default: false)
  parameter(:show_legend, "Show legend?", "When enabled, legend will be displayed on chart", :boolean, default: true)

  def build_results!
    setup_targets!

    s_time, e_time = start_and_end_time

    machine_id_groups.each do |machine_id_group|
      machine_throughput_series = push_series(machine_id_group[:series_name], machine_id_group[:series_name])
      machine_throughput_series[:dash_array] = 0
      machine_throughput_series[:data_labels_toggle] = data_labels_toggle

      throughput_per_machine = MachineKpi.throughput_between(s_time, e_time, Timeframe::HOUR_BY_HOUR, machine_id_group[:machine_ids])

      throughput_per_machine.each do | k,v |
        bucket_label = "#{k.strftime("%H:%M")} - #{(k + 1.hour).strftime("%H:%M")}"
        push_point(machine_id_group[:series_name], bucket_label, v[:count])
      end
    end

    if target_throughput && target_throughput > 0 && dataset.length > 0
      throughput_target_series = push_series('throughput_target', 'Throughput Target')
      throughput_target_series[:data_labels_toggle] = false
      throughput_target_series[:dash_array] = 5
      throughput_target_series[:color] = "#FF0000"
      dataset[0][:data].each do |point|
        push_point(throughput_target_series[:id], point[:x], target_throughput)
      end
    end
  end

  def setup_targets!
    @_machine_id_groups = parsed_targets.map do |target|
      if target.is_a?(MachineGroup)
        if split_machine_groups
          target.machines.map do |sub_target|
            {
              series_name: MachineDashboardHelpers.target_to_name(sub_target),
              machine_ids: [sub_target.id]
            }
          end
        else
          {
            series_name: MachineDashboardHelpers.target_to_name(target),
            machine_ids: target.machine_ids
          }
        end
      else
        {
          series_name: MachineDashboardHelpers.target_to_name(target),
          machine_ids: [target.id]
        }
      end
    end.flatten.uniq
  end

  def sort_series
    false
  end

  def machine_id_groups
    @_machine_id_groups ||= []
  end

  def split_machine_groups
    @parameters[:split_machine_groups]
  end

  def target_throughput
    @parameters[:target_throughput]
  end

  def data_labels_toggle
    @parameters[:data_labels_toggle]
  end
end