import * as React from "react";
import {observer} from "mobx-react-lite";
import {Box, makeStyles, Typography} from "@material-ui/core";
import {t} from "@lingui/macro";
import {MachineManualMap, MachineTypeModel, ManualLanguage, ManualType} from "../../../stores/MachineTypeStore";
import {ButtonField, SelectField} from "@rx-suite/ui";
import {useMemo} from "react";
import {GetApp} from "@material-ui/icons";
import {useSnackbar} from "notistack";

const useStyles = makeStyles(theme => ({
    label: {
        width: '250px'
    },
    languageSelect: {
        width: '175px'
    }
}));

type MachineManualDownloadProps = {
    label: string;
    model: MachineTypeModel;
    manualType: ManualType;
}

export const MachineManualDownload: React.FunctionComponent<MachineManualDownloadProps> = observer(({ label, model, manualType }) => {
    const classes = useStyles()

    const {enqueueSnackbar} = useSnackbar();

    const availableLanguages = useMemo(() => {
        if(model.machine_manuals === null){
            return [];
        }

        const base: {label: string, value: ManualLanguage}[] = [
            {label: 'English (EN)', value: 'en'},
            {label: 'Deutsch (DE)', value: 'de'},
            {label: 'Español (ES)', value: 'es'},
            {label: 'Français (FR)', value: 'fr'},
            {label: 'Italiano (IT)', value: 'it'},
            {label: '日本語 (JP)', value: 'ja'},
            {label: 'Polski (PL)', value: 'pl'},
            {label: 'Português (PT)', value: 'pt'}
        ];

        const modelAvailableLanguages = Object.keys(model.machine_manuals[manualType] as MachineManualMap);

        return base.filter(lang => modelAvailableLanguages.includes(lang.value));
    }, [model])

    const [selectedLanguage, setSelectedLanguage] = React.useState<ManualLanguage>(availableLanguages[0].value);
    const [isDownloading, setIsDownloading] = React.useState<boolean>(false);

    const download = React.useCallback( async () => {
        setIsDownloading(true)

        try {
            await model.getManual(manualType, selectedLanguage);
            enqueueSnackbar(t`Download successful`, {variant: "success"});
        } catch (e) {
            console.error(e)
            enqueueSnackbar(t`Download failed`, {variant: "error"});
        }

        setIsDownloading(false)
    }, [model, manualType, selectedLanguage]);

    return (
        <Box display={'flex'} alignItems={"center"}>
            <Typography className={classes.label} variant={'h6'}>{label}</Typography>
            <Box>
                <SelectField
                    options={availableLanguages}
                    onChange={(_, val) => {
                        setSelectedLanguage(val)
                    }}
                    value={selectedLanguage}
                    className={classes.languageSelect}
                />
                <ButtonField
                    text={t`Download`}
                    onClick={download}
                    loading={isDownloading}
                    startIcon={<GetApp/>}
                />
            </Box>
        </Box>
    )
})

