import * as React from "react";
import {observer} from "mobx-react-lite";
import {MachineModel} from "../../../stores/MachineStore";
import {ButtonField, makeFormattedDateTimeColumn, ModelTable, SelectField} from "@rx-suite/ui";
import {ConsumableRecordModel, ConsumableRecordStore} from "../../../stores/ConsumableRecordStore";
import {t} from "@lingui/macro";
import Box from "@material-ui/core/Box";
import {MachineConsumableDetailsDialog} from "./MachineConsumableDetailsDialog";
import {Filter, ReactTableFunction} from "react-table";
import {createStyles, makeStyles, Theme, Typography} from "@material-ui/core";
import {MachineTypeModel} from "../../../stores/MachineTypeStore";

type MachineConsumablesViewProps = {
    machine: MachineModel;
    machineType: MachineTypeModel;
}

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        detailsButton: {
            margin: '0px'
        }
    })
);

export const MachineConsumablesView: React.FunctionComponent<MachineConsumablesViewProps> = observer(({machine, machineType}) => {
    const classes = useStyles();

    const [dialogConsumableRecord, setDialogConsumableRecord] = React.useState<ConsumableRecordModel|null>(null);

    const consumableColumns = [
        {
            Header: t`Tool`,
            accessor: (consumableRecord: ConsumableRecordModel) => consumableRecord,
            id: 'name',
            Cell: (cellInfo: {value: ConsumableRecordModel}) => {
                return cellInfo.value.resolved_name;
            },
            Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction }) => (
                <SelectField
                    options={ConsumableRecordStore.optionsForMachineType(machineType)}
                    onChange={(name, value) => onChange(value)}
                    value={filter ? filter.value : ""}
                    clearable
                />
            ),
            strictMatchFilter: true,
            filterable: true,
            sortable: true,
        },
        {
            Header: 'Use Count',
            accessor: 'current_use_count',
            filterable: false,
            sortable: true
        },
        {
            Header: 'Max Use',
            accessor: 'max_use_count',
            filterable: false,
            sortable: true
        },
        makeFormattedDateTimeColumn('first_use_date', 'First Use', 'dateTime'),
        makeFormattedDateTimeColumn('last_use_date', 'Last Use', 'dateTime'),
        {
            Header: 'Status',
            accessor: 'retired',
            Cell: (cellInfo: {value: boolean}) => {
                if (cellInfo.value) {
                    return <Box color={"red"}>Retired</Box>;
                }
                return <Box color={"green"}>In Use</Box>;
            },
            filterable: false,
            sortable: true
        },
        {
            accessor: (consumableRecord: ConsumableRecordModel) => consumableRecord,
            id: '_details',
            Cell: (cellInfo: {row: { name: string; }, value: ConsumableRecordModel}) => {
                return (
                    <ButtonField
                        variant={'text'}
                        text={<Typography style={{textDecoration: 'underline', textTransform: 'none'}}>{t`view details`}</Typography>}
                        className={classes.detailsButton}
                        onClick={() => setDialogConsumableRecord(cellInfo.value)}
                    />
                );
            },
            filterable: false,
            sortable: false
        }
    ];

    return (
        <React.Fragment>
            <ModelTable
                suppressWrapper
                suppressActionColumn
                store={ConsumableRecordStore}
                columns={consumableColumns}
                defaultSorted={[{id: "last_use_date", desc: true}]}
                defaultFiltered={[{id: "machine_id", value: machine.id}]}
            />
            <MachineConsumableDetailsDialog consumableRecord={dialogConsumableRecord} setConsumableRecord={setDialogConsumableRecord}/>
        </React.Fragment>
    );
})