import {attribute, DataStore, model, Model} from '@rx-suite/data';

import {t} from "@lingui/macro";
import {computed} from "mobx";

@model({
    eager: true,
    transportLayer: "/api/machine_types",
    routingInfo: {
        update: "/settings/machine_types/:id/edit",
        read: "/settings/machine_types/:id"
    }
})
export class MachineTypeModel extends Model {
    @attribute({label: t`Name`})
    name: string = "";

    @attribute({label: t`Display Name`})
    display_name: string = "";

    @attribute({label: t`Category`})
    category: string = "";

    @attribute({ label: t`Machine Process ID` })
    machine_process_id: number | null = null;

    @attribute()
    image_url: string = "";

    @attribute()
    interfaces: InterfaceMap = {
        machine_state: true,
        job_state: true,
        error: true,
        maintenance: true,
        process_data: true,
        event: true,
        calibration: true
    }

    @attribute()
    machine_manuals: MachineManualMap | null = null

    @attribute()
    has_consumable_data: boolean = false

    @computed
    get has_machine_manuals() {
        return this.machine_manuals !== null;
    }

    @computed
    get has_maintenance_manuals() {
        return this.machine_manuals !== null && this.machine_manuals.maintenance !== undefined;
    }

    @computed
    get has_operating_manuals() {
        return  this.machine_manuals !== null && this.machine_manuals.operating !== undefined;
    }

    @attribute()
    manual_languages: any = null;

    @computed
    get isBoxCoater(): boolean {
        return this.category === 'box_coater';
    }

    @computed
    get allowsEditMachineProcess(): boolean {
        const allowsEdit = ['other']
        return allowsEdit.includes(this.name);
    }

    getManual = async (manualType: ManualType, manualLanguage: ManualLanguage) => {
        const response = await this.store.transportLayer.fetch(`/${this.id}/download_manual/?manualType=${manualType}&manualLanguage=${manualLanguage}`, {
            method: 'GET',
            headers: {
                'Content-Type': 'application/pdf'
            },
            responseType: 'blob'
        })

        downloadPdfFromBlob(await response.blob(), `${this.display_name.replaceAll("-", "_")}_${manualType}_${manualLanguage}.pdf`);
    }

    constructor() {
        super();
        super.setup();
    }
}

function downloadPdfFromBlob(blob: Blob, filename: string) {
    const url = window.URL.createObjectURL(new Blob([blob]));
    const link = document.createElement('a');
    link.href = url;
    link.setAttribute('download', filename);
    document.body.appendChild(link);
    link.click();
    link.parentNode.removeChild(link);
}

type InterfaceMap = {
    machine_state: boolean;
    job_state: boolean;
    error: boolean;
    maintenance: boolean;
    process_data: boolean;
    event: boolean;
    calibration: boolean;
    [key: string] : boolean;
}

export type MachineManualMap = {
    maintenance?: {
        [key: string] : {name: string};
    },
    operating?: {
        [key: string] : {name: string};
    }
}

export type ManualType = 'operating' | 'maintenance';
export type ManualLanguage = 'en' | 'de' | 'es' | 'fr' | 'it' | 'ja' | 'pl' | 'pt';

export const MachineTypeStore = MachineTypeModel.store as DataStore<MachineTypeModel>;
