module MachineKpiCalculator
  class IdleTimeRuns < EnergyUsageBreakdown
    include IntervalBuilder
    include BucketCalculator

    class << self
      def get_for_period_and_machines(start_time, end_time, machine_ids, options)
        kpi = IdleTimeRuns.new(start_time, end_time, KpiHelpers::Timeframe::ALL, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :idle_runs,
          handler: :idle_runs,
          params: {}
        }
      ]
    end

    def idle_runs
      result_builder(start_time, end_time) do |interval_start, interval_end|
        interval_results = {}
        machine_ids.each do |machine_id|
          machine = Machine.find(machine_id)
          interval_results[machine_id] = interval_idle_times(machine, interval_start, interval_end)
        end
        interval_results
      end
    end

    def interval_idle_times(machine, interval_start, interval_end)
      #tf we are in the archive b
      if interval_end.min == 0 || interval_end.min == 30
        interval_end = interval_end.advance(minutes: 29, seconds: 59)
      end
      state_at_start = machine.state_prior_to_start(interval_start)
      machine_states = MachineState.where(
        time: interval_start..interval_end,
        machine_sn: machine.sn
      ).order(time: :asc).to_a

      previous_state_time = nil
      unless state_at_start.nil?
        previous_state_time = state_at_start.time
        state_at_start.time = interval_start
        machine_states = machine_states.unshift(state_at_start)
      end

      unproductive_state_durations = []

      start_time = nil
      unproductive_energy = 0
      latest_time = interval_end > Time.zone.now ? Time.zone.now : interval_end
      flag_for_merge_with_results = true
      flag_for_last_state_was_merge_with_results = false

      machine_states.each_with_index do |machine_state, i|
        is_last_state = machine_state == machine_states.last
        is_first_state = machine_state == machine_states.first
        previous_state = is_first_state ? nil : machine_states[i-1]
        not_unproductive_states = %w[processing off] # processing is productive, off is considered neither productive or unproductive
        unproductive_state = !not_unproductive_states.include?(machine_state.state)
        machine_type = machine.machine_type

        if unproductive_state

          # If we have previous results,
          # we need to check if the first state of this time interval needs to be merged with the end of the previous results
          # This happens when we are in the archive and we have an end state that is unproductive, and in the current
          # interval we start with an unproductive state also.
          # If interval start is smaller then results end, that means that results end is growing, so keep going until
          # productive time
          if flag_for_merge_with_results &&
            @results[machine.id] &&
            @results[machine.id].last &&
            ((interval_start - @results[machine.id].last[:end] == 1.seconds) ||
              interval_start < @results[machine.id].last[:end])

            # Get the energy consumed by the state for its duration
            state_duration = get_state_duration(machine_state, is_last_state, latest_time)
            # since its the first state, it is probably continuing from a state in the previous interval, so we need to
            # remove that from the calculated state duration
            if is_first_state
              duration_to_remove = @results[machine.id].last[:end] - previous_state_time
              state_duration -= duration_to_remove
            end

            energy_consumption = get_state_consumption(machine_state.state, machine_type, state_duration)

            @results[machine.id].last[:energy] += energy_consumption
            @results[machine.id].last[:duration] += state_duration
            flag_for_last_state_was_merge_with_results = true
            next
          end

          # Set state time as start time if first state or previous state is processing
          if is_first_state || not_unproductive_states.include?(previous_state.state)
            start_time = machine_state.time
          end

          # Get the energy consumed by the state for its duration
          state_duration = get_state_duration(machine_state, is_last_state, latest_time)
          energy_consumption = get_state_consumption(machine_state.state, machine_type, state_duration)
          unproductive_energy += energy_consumption
        end

        # Skip over a processing state if the previous state is also processing.
        next if !!previous_state && !unproductive_state && not_unproductive_states.include?(previous_state.state)

        if !unproductive_state || is_last_state && unproductive_state

          if flag_for_last_state_was_merge_with_results
            @results[machine.id].last[:end] = machine_state.time
            flag_for_last_state_was_merge_with_results = false
            flag_for_merge_with_results = false
            next
          end

          next if is_first_state
          # Edge Case: Use latest time as the state end time if the last state we get is the machine's last state
          end_time = is_last_state && unproductive_state ? latest_time : machine_state.time

          # Only consider idle time if it is greater than 30 minutes
          if end_time - start_time >= 1800
            unproductive_state_durations << build_idle_time(start_time, end_time, unproductive_energy)
          end

          unproductive_energy = 0
        end

      end

      unproductive_state_durations
    end

    def get_state_duration(machine_state, is_last_state, latest_time)
      if is_last_state
        latest_time - machine_state.time
      else
        machine_state.calculated_duration
      end
    end

    def build_idle_time(unproductive_start, productive_start, unproductive_energy)
      {
        start: unproductive_start,
        end: productive_start,
        duration: productive_start - unproductive_start,
        energy: unproductive_energy  # kWh
      }
    end

    def append_to_result_all(interval_data)
      interval_data.each do |machine_id, idle_times|
        @results[machine_id] = [] if @results[machine_id].nil?
        @results[machine_id] += idle_times
      end
    end

    def build_results
      # Probably should return a flat array of all batches over the duration ? Not sure if it's useful however
      raise NotImplementedError
    end

    def build_results_per_machine
      data[:idle_runs]
    end

    def wrap_with_count
      if aggregate_machines?
        @results.each_with_index do |bucket, i|
          @results[i][:duration] = {count: bucket[:duration]}
        end
      else
        @results.each do |machine_id, hash|
          hash.each_with_index do |bucket, i|
            @results[machine_id][i][:duration] = {count: bucket[:duration]}
          end
        end
      end
    end

    def as_percent(results)
      raise NotImplementedError
    end
  end
end
